# ../scripts/custom/gg_scoreboard/gg_scoreboard.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# EventScripts Imports
#   ES
from es import getUseridList
from es import ServerVar
#   Gamethread
from gamethread import delayed

# SPE Imports
from spe import detourFunction
from spe import getIndexOfEntity
from spe import HookAction
from spe import HookType
from spe import parseINI
from spe import undetourFunction

# SPE Tools Imports
#   Player
from spe.tools.player import SPEPlayer

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo
#   Players
from gungame51.core.players.shortcuts import Player
from gungame51.core.players.shortcuts import add_attribute_callback
from gungame51.core.players.shortcuts import remove_callbacks_for_addon
#   Weapons
from gungame51.core.weapons.shortcuts import get_level_multikill

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = AddonInfo()
info.name = 'gg_scoreboard'
info.title = 'GG Scoreboard' 
info.author = 'Satoon101' 
info.version = '2.3'

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()

# =============================================================================
# >> GLOBAL VARIABLES
# =============================================================================
gg_scoreboard_multikill = ServerVar('gg_scoreboard_multikill')


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    '''Called when the script is loaded'''

    # Register the level attribute callback
    add_attribute_callback('level', level_callback, info.name)

    # Register the multikill attribute callback
    add_attribute_callback('multikill', multikill_callback, info.name)

    # Parse the INI file for SPE
    parseINI('gungame51/scripts/custom/gg_scoreboard/signatures.ini')

    # Register the IncrementscoreCount detour
    detourFunction('IncrementFragCount', HookType.Pre, increment_score)

    # Register the IncrementDeathCount detour
    detourFunction('IncrementDeathCount', HookType.Pre, increment_deaths)

    # Set all player scores
    set_all_player_scores()


def unload():
    '''Called when the script is unloaded'''

    # Unregister the level and multikill callbacks
    remove_callbacks_for_addon(info.name)

    # Unregister the IncrementscoreCount detour
    undetourFunction('IncrementFragCount', HookType.Pre, increment_score)

    # Unregister the IncrementDeathCount detour
    undetourFunction('IncrementDeathCount', HookType.Pre, increment_deaths)


# =============================================================================
# >> DETOUR CALLBACKS
# =============================================================================
def increment_score(args):
    '''Called when the player earns a score'''

    # Delay 1 tick, then reset the player's 
    delayed(0, reset_score, args[1])

    # Allow the function to continue
    return (HookAction.Continue, 0)


def increment_deaths(args):
    '''Called when the player earns a death'''

    # Does the player's deaths need to be reset?
    if int(gg_scoreboard_multikill):

        # Delay 1 tick, then reset the player's deaths
        delayed(0, reset_deaths, args[1])

    # Allow the function to continue
    return (HookAction.Continue, 0)


# =============================================================================
# >> DELAY CALLS
# =============================================================================
def reset_score(pointer):
    '''Resets a player's kills after they have changed'''

    # Get the player's index
    index = getIndexOfEntity(pointer)

    # Is the player still on the server?
    if not index:

        # If not, return
        return

    # Get the player's SPEPlayer instance
    player = SPEPlayer.get_instance_from_index(index)

    # Get the player's Player instance
    ggPlayer = Player(player.userid)

    # Reset the player's kills
    level_callback('level', ggPlayer.level, ggPlayer)


def reset_deaths(pointer):
    '''Resets a player's deaths after they have changed'''

    # Get the player's index
    index = getIndexOfEntity(pointer)

    # Is the player still on the server?
    if not index:

        # If not, return
        return

    # Get the player's SPEPlayer instance
    player = SPEPlayer.get_instance_from_index(index)

    # Get the player's Player instance
    ggPlayer = Player(player.userid)

    # Reset the player's deaths
    multikill_callback('multikill', ggPlayer.multikill, ggPlayer)


# =============================================================================
# >> ATTRIBUTE CALLBACKS
# =============================================================================
def level_callback(name, value, ggPlayer):
    '''Called when the given player's level is being changed'''

    # Set the player's kills to their level
    SPEPlayer(ggPlayer.userid).kills = value


def multikill_callback(name, value, ggPlayer):
    '''Called when the given player's multikill is being changed'''

    # Get the value of gg_scoreboard_multikill
    multikill = int(gg_scoreboard_multikill)

    # Does the player's death total need changed?
    if not multikill:

        # If not, simply return
        return

    # Get the SPEPlayer instance
    spe_player = SPEPlayer(ggPlayer.userid)

    # Does the player's death total need changed to their multikill?
    if multikill == 1:

        # Set the player's death total to their multikill
        spe_player.deaths = value

    # Does the player's death total need changed to their remaining kills?
    else:

        # Get the player's level's multikill value
        level_multikill = get_level_multikill(ggPlayer.level)

        # Set the player's death total to their remaining kills
        spe_player.deaths = level_multikill - value


# =============================================================================
# >> GAME EVENTS
# =============================================================================
def player_spawn(event_var):
    '''Called every time a player spawns'''

    # Is the player on a team?
    if int(event_var['es_userteam']):

        # Set the player's score
        set_player_score(event_var['userid'])


# =============================================================================
# >> CUSTOM/HELPER FUNCTIONS
# =============================================================================
def set_all_player_scores():
    '''Sets all player scores when gg_scoreboard is loaded'''

    # Loop through all players currently on the server
    for userid in getUseridList():

        # Set the player's score
        set_player_score(userid)


def set_player_score(userid):
    '''Sets the score for the given player'''

    # Get the Player instance
    ggPlayer = Player(userid)

    # Set the player's score
    level_callback('level', ggPlayer.level, ggPlayer)

    # Set the player's death total
    multikill_callback('multikill', ggPlayer.multikill, ggPlayer)
